#ifndef EXAMPLE_H
#define EXAMPLE_H
#include <qapplication.h>
#include <qpopupmenu.h>
#include <qmainwindow.h>
#include <qintdict.h>
#include <qcanvas.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <stdlib.h>
#include <time.h>

static const int imageRTTI = 999999;
const int ghost_rtti = 1234;
//........................................................ Klasse ImageItem
class ImageItem: public QCanvasRectangle {
 public:
    ImageItem(QImage img, QCanvas *c) : QCanvasRectangle(c), image(img) {
       setSize( image.width(), image.height() );
       pixmap.convertFromImage(image, OrderedAlphaDither);
    }
    int rtti () const { return imageRTTI; }
    bool hit( const QPoint &p ) const {
       int ix = p.x()-int(x()),
           iy = p.y()-int(y());
       if ( !image.valid( ix, iy) )
          return false;
       QRgb pixel = image.pixel( ix, iy );
       return qAlpha( pixel ) != 0;
    }
 protected:
    void drawShape( QPainter &p ) { p.drawPixmap(int(x()), int(y()), pixmap); }
 private:
    QImage   image;
    QPixmap  pixmap;
    int      ix, iy;
};
//..................................................... Klasse FigureEditor
class FigureEditor : public QCanvasView {
    Q_OBJECT
public:
    FigureEditor( QCanvas& c, QWidget* par=0 ) : QCanvasView( &c, par ) { }
    void clear() {
       QCanvasItemList list = canvas()->allItems();
       QCanvasItemList::Iterator it = list.begin();
       for (; it != list.end(); ++it)
         if ( *it )
            delete *it;
    }
protected:
    void contentsMousePressEvent(QMouseEvent* e) {
       QCanvasItemList l=canvas()->collisions(e->pos());
       for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it) {
          if ( (*it)->rtti() == imageRTTI ) {
              ImageItem *item = (ImageItem*)(*it);
              if ( !item->hit( e->pos() ) )
                 continue;
          }
          moving = *it;
          moving_start = e->pos();
          return;
       }
       moving = 0;
    }
    void contentsMouseMoveEvent(QMouseEvent* e) {
       if ( moving ) {
          moving->moveBy(e->pos().x() - moving_start.x(),
                         e->pos().y() - moving_start.y());
          moving_start = e->pos();
          canvas()->update();
       }
    }
signals:
    void status(const QString&);
private:
    QCanvasItem* moving;
    QPoint       moving_start;
};
//....................................................... Klasse FlyingPic
class FlyingPic : public QCanvasSprite {
 public:
    FlyingPic::FlyingPic(QCanvas* canvas) : QCanvasSprite(0,canvas) {
       static QCanvasPixmapArray ghost("penguin.png");
       setSequence( &ghost );
       setAnimated( true );
       initPos();
    }
    void advance(int);
    int rtti() const { return ghost_rtti; }
 private:
    void initPos() {
       initSpeed();
       int trial=1000;
       do {
          move( rand()%canvas()->width(), rand()%canvas()->height());
          advance(0);
       } while ( trial-- && xVelocity()==0.0 && yVelocity()==0.0 );
    }
    void initSpeed() {
       const double speed = 4.0;
       double d = (double)(rand()%1024) / 1024.0;
       setVelocity( d*speed*2-speed, (1-d)*speed*2-speed );
    }
};
//............................................................. Klasse Main
class Main : public QMainWindow {
    Q_OBJECT
public:
    Main( QString, QString, QCanvas&, QWidget* par=0 );
private slots:
    void newView() {
       Main *m = new Main( butterfly_fn, picture_fn, canvas, 0 );
       qApp->setMainWidget( m );
       m->show();
       qApp->setMainWidget( 0 );
    }
    void clear() { editor->clear(); }
    void init() {
       clear();
       for ( int i=0; i<4; i++) {
          addPicture();
          addChair();
          addButterfly();
       }
    }
    void addSprite() {
       QCanvasItem* i = new FlyingPic( &canvas );
       i->setZ(rand()&255);
       i->show();
    }
    void addCircle() {
       QCanvasPolygonalItem* i = new QCanvasEllipse( 50, 50,&canvas);
       i->setBrush( QColor(rand()&255,rand()&255,rand()&255) );
       moveItem( (QCanvasPolygonalItem *)i, 50, 50 );
    }
    void addChair() {
       QCanvasPolygon* i = new QCanvasPolygon(&canvas);
       QPointArray pa;
       pa.putPoints(0,14, 30,0, 100,0, 100,180, 90,180, 90,110, 70,140, 70,210,
                          60,210, 60,140, 10,140, 10,210, 0,210, 0,140, 30,100);
       i->setPoints( pa );
       i->setBrush( QColor(rand()&255,rand()&255,rand()&155) );
       moveItem( (QCanvasPolygon *)i, 100, 210 );
    }
    void addLine() {
       QCanvasLine* i = new QCanvasLine( &canvas );
       int r, b;
       i->setPoints( rand()%canvas.width(), rand()%canvas.height(),
                     r=rand()%canvas.width(), b=rand()%canvas.height() );
       i->setPen( QPen( QColor( rand()&255, rand()&255, rand()&255 ), 6) );
       moveItem( (QCanvasLine *)i, r, b );
    }
    void addRectangle() {
       int  x = rand() % canvas.width(),
            y = rand() % canvas.height(),
            w = rand() % (canvas.width() - x ),
            h = rand() % (canvas.height() - y );
       QCanvasPolygonalItem *i = new QCanvasRectangle( x, y, w, h, &canvas );
       i->setBrush( QColor( rand()&255, rand()&255, rand()&255 ) );
       i->setPen( QPen( QColor( rand()&255, rand()&255, rand()&255 ), 6) );
       moveItem((QCanvasPolygonalItem *)i, canvas.width()-w, canvas.height()-h);
    }
    void addPicture() {
       if ( !picture_fn.isEmpty() ) {
          static QImage *img;
          displayImage( img, picture_fn );
       }
    }
    void addButterfly() {
       if ( !butterfly_fn.isEmpty() ) {
          static QImage *img;
          displayImage( img, butterfly_fn );
       }
    }
    void enlarge() { canvas.resize(canvas.width()*4/3, canvas.height()*4/3); }
    void shrink()  { canvas.resize(canvas.width()*3/4, canvas.height()*3/4); }
private:
    QCanvas      &canvas;
    FigureEditor *editor;
    QString      butterfly_fn, picture_fn;
    
    void moveItem( QCanvasItem *item, int offX, int offY ) {
       item->move( rand()%(canvas.width()-offX), rand()%(canvas.height()-offY) );
       item->setZ( rand()&255 );
       item->show(); 
    }
    void displayImage( QImage *img, QString fileName ) {
       if ( !img ) {
          img = new QImage[4];
          img[0].load( fileName );
          int w = img[0].width(), h = img[0].height();
          img[1] = img[0].smoothScale( int(w*0.75), int(h*0.75) );
          img[2] = img[0].smoothScale( int(w*0.5 ), int(h*0.5 ) );
          img[3] = img[0].smoothScale( int(w*0.25), int(h*0.25) );
       }
       QCanvasPolygonalItem* i = new ImageItem( img[rand()%4], &canvas );
       i->move( rand()%(canvas.width()-img->width() ),
                rand()%(canvas.height()-img->height() ) );
       i->setZ( rand()&255 + 250 );
       i->show();
    }
};
#endif
