#include <qapplication.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qpainter.h>
#include <qdom.h>

#define  MAX_VALUES  10000

//.............................................................. DataHandler
class DataHandler : public QDomDocument
{
public:
    DataHandler() { }
    ~DataHandler() { }

    int    width()    { return w;       }
    int    height()   { return h;       }
    int    nrValues() { return valueNr; }
    double getXValue( int index ) { return valuePair[index].x; }
    double getYValue( int index ) { return valuePair[index].y; }

    bool parse( QDomElement docElem ) { 
       errorMsg = "";
       state = StateInit;
       w = 600;
       h = 400;
       valueNr = 0;
       parseTree( docElem );
       return errorMsg.isEmpty();
    }

   void parseTree( QDomElement docElem ) {
       QDomNode n = docElem;
       while( !n.isNull() ) {
          QDomElement e = n.toElement();
          if( !e.isNull() ) {
              startElement( e );
              if ( !e.firstChild().isNull() )
                 parseTree( e.firstChild().toElement() );
              endElement();
          }
          n = n.nextSibling();
       }
    } 

    void startElement( QDomElement e ) {
       QString nodeName = e.nodeName();
       if ( nodeName == "function" && state == StateInit ) {
          state = StateDocument;
       } else if ( nodeName == "config" && state == StateDocument ) {
          state = StateConfig;
          QString tmp;
          tmp = e.attributeNode( "width" ).value();
          if ( (w = tmp.toInt(0) ) < 100 )
             errorMsg = "Illegal width";
          tmp = e.attributeNode( "height" ).value();
          if ( (h = tmp.toInt(0) ) < 100 )
             errorMsg = "Illegal height";
       } else if ( nodeName == "value" && state == StateDocument ) {
          state = StateValue;
          QString tmp;
          bool  ok = true;
          tmp = e.attributeNode( "x" ).value();
          valuePair[valueNr].x = tmp.toDouble(&ok);
          if ( !ok )
             errorMsg = "Illegal or missing x-Value";
          tmp = e.attributeNode( "y" ).value();
          valuePair[valueNr].y = tmp.toDouble(&ok);
          if ( !ok )
             errorMsg = "Illegal or missing x-Value";
          valueNr++;
       }
    }

    void endElement( void ) {
       switch ( state ) {
          case StateConfig: state = StateDocument; break;
          case StateValue:  state = StateDocument; break;
          default:                                 break;
       }
    }

    QString errorString() { return errorMsg; }

private:
    struct {  double  x, y; } valuePair[MAX_VALUES];
    QString     errorMsg;
    int         w, h, valueNr;

    enum State { StateInit, StateDocument, StateConfig, StateValue };
    State state;
};

//................................................................... Plotter
class Plotter : public QWidget
{
public:
    Plotter( DataHandler handler, QWidget *p=0, const char *n=0 )
                                                         : QWidget( p, n ) {
       data = handler;
       resize( data.width(), data.height() );
       repaint( true );
   }

protected:
    virtual void paintEvent( QPaintEvent *) {
       int  i;
       QPainter p( this );
       p.setBackgroundColor( Qt::white );

       double minx = data.getXValue(0),
              maxx = data.getXValue(0),
              miny = data.getYValue(0),
              maxy = data.getYValue(0);
       for ( i=1; i<data.nrValues(); i++ ) {
          double x = data.getXValue(i),
                 y = data.getYValue(i);
          if ( x < minx ) minx = x;
          if ( x > maxx ) maxx = x;
          if ( y < miny ) miny = y;
          if ( y > maxy ) maxy = y;
       }

       double  xFactor = width() / (maxx -minx),
               yFactor = height() / (maxy -miny),
               xp = ( data.getXValue(0) - minx ) * xFactor,
               yp = ( maxy - data.getYValue(0) ) * yFactor;
       p.moveTo( int(xp), int(yp) );
       p.setPen( QPen( Qt::black, 2 ) );
       p.setBrush( Qt::black );
       p.drawEllipse( int(xp-1), int(yp-1), 2, 2 );
 
       for ( i=1; i<data.nrValues(); i++ ) {
          xp = ( data.getXValue(i) - minx ) * xFactor;
          yp = ( maxy - data.getYValue(i) ) * yFactor;
          p.setPen( QPen( Qt::blue, 2 ) );
          p.lineTo( int(xp), int(yp) );
          p.setPen( QPen( Qt::black, 2 ) );
          p.drawEllipse( int(xp-2), int(yp-2), 4, 4 );
       }
    }

private:
    DataHandler data;
};

//...................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );

    QString  msg;
    if ( argc != 2 ) {
       msg.sprintf( "usage:  %s xml-file", argv[0] );
       QMessageBox::information( 0, "Falscher Aufruf", msg, QMessageBox::Ok );
       return 1;
    }
    QFile file( argv[1] );
    if ( !file.open( IO_ReadOnly ) ) {
       msg.sprintf( "cannot open file `%s'", argv[1] );
       QMessageBox::information( 0, "file open error", msg, QMessageBox::Ok );
       return 1;
    }
    DataHandler   handler;
    if ( !handler.setContent( &file ) ) {
        file.close();
        return -1;
    }
    file.close();

    bool ok = handler.parse( handler.documentElement() );
    if ( !ok ) {
       QMessageBox::critical( 0, a.tr( "Wrong config or values" ),
                              a.tr( handler.errorString() ) );
       return -1;
    }

    Plotter plot( handler );
    plot.setCaption( "XML-Functionplotter" );
    a.setMainWidget( &plot );
    plot.show();
    return a.exec();
}
