#include <qapplication.h>
#include <qwidget.h>
#include <qscrollview.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qkeycode.h>
#include <qimage.h>
#include <stdio.h>

/* XPM */
static const char *image_xpm[] = {
"17 15 9 1",
"        c #7F7F7F",
".       c #FFFFFF",
"X       c #00B6FF",
"o       c #BFBFBF",
"O       c #FF6C00",
"+       c #000000",
"@       c #0000FF",
"#       c #6CFF00",
"$       c #FFB691",
"             ..XX",
" ........o   .XXX",
" .OOOOOOOo.  XXX+",
" .O@@@@@@+++XXX++",
" .O@@@@@@O.XXX+++",
" .O@@@@@@OXXX+++.",
" .O######XXX++...",
" .O#####XXX++....",
" .O##$#$XX+o+....",
" .O#$$$$$+.o+....",
" .O##$$##O.o+....",
" .OOOOOOOO.o+....",
" ..........o+....",
" ooooooooooo+....",
"+++++++++++++...."
};

//................................................. class ImageIconProvider
class ImageIconProvider : public QFileIconProvider {

public:
    ImageIconProvider( QWidget *parent=0, const char *name=0 )
               : QFileIconProvider( parent, name ), imagepm(image_xpm) {
       fmts = QImage::inputFormats();
    }

    const QPixmap * pixmap( const QFileInfo &fi ) {
       QString ext = fi.extension().upper();
       if ( fmts.contains(ext) )
          return &imagepm;
       else
          return QFileIconProvider::pixmap(fi);
    }

private:
    QStrList fmts;
    QPixmap  imagepm;
};

//........................................................... class MyLabel
class MyLabel : public QLabel {
    Q_OBJECT
public:
    MyLabel( QWidget* parent = 0 ) : QLabel( parent) {
       setMouseTracking( true );
    }
protected:
    virtual void mouseMoveEvent( QMouseEvent *e ) {
       emit pos( e->pos().x(), e->pos().y() );
    }
signals:
    void pos( int x, int y);
};

//...................................................... class MyScrollView
class MyScrollView : public QScrollView {
    Q_OBJECT
public:
    MyScrollView( QWidget* parent = 0 ) : QScrollView( parent) {
       l = new MyLabel( viewport() );
       addChild( l );                                                       
       QObject::connect( l, SIGNAL( pos( int, int ) ),
                         this, SLOT( mouseMoved( int, int ) ) );
    }
    void setPixmap( QPixmap pm ) {
       l->setPixmap( pm );
    }

private slots:
    void mouseMoved( int x, int y ) {
       emit pos( x, y );
    }

signals:
    void pos( int x, int y);

private:
    MyLabel  *l;
};

//....................................................... class ImageViewer
class ImageViewer : public QWidget {
    Q_OBJECT
public:
    ImageViewer( QWidget *parent=0, const char *name=0, int wFlags=0 );

    void loadImage( QString fileName ) {
       filename = fileName;
       if ( filename ) {
          QApplication::setOverrideCursor( waitCursor );
          if ( image.load(filename, 0) &&  pm.convertFromImage(image) ) {
             setCaption( filename );
             ms->setPixmap( pm );
          }
          QApplication::restoreOverrideCursor();
       }
       updateStatus();
       setMenuItemFlags();
    }

private slots:
    void to1Bit()  { toBitDepth(1);  }
    void to8Bit()  { toBitDepth(8);  }
    void to32Bit() { toBitDepth(32); }

    void toBitDepth( int d ) {
       QApplication::setOverrideCursor( waitCursor );
       image = image.convertDepth(d);
       if ( pm.convertFromImage(image) )
          ms->setPixmap( pm );
       QApplication::restoreOverrideCursor();
       updateStatus();
       setMenuItemFlags();
    }

    void hFlip()  { setImage(image.mirror(true, false)); }
    void vFlip()  { setImage(image.mirror(false, true)); }
    void rot180() { setImage(image.mirror(true, true));  }
    void swapRgb()   { setImage(image.swapRGB());   }
    void invertPix() { 
       QApplication::setOverrideCursor( waitCursor );
       image.invertPixels();
       if ( pm.convertFromImage(image) )
          ms->setPixmap( pm );
       QApplication::restoreOverrideCursor();
    }

    void openFile() {
       QString newfilename = QFileDialog::getOpenFileName();
       if ( !newfilename.isEmpty() )
          loadImage( newfilename ) ;
    }
    void saveImage( int item ) {
        const char* fmt = saveimage->text(item);
        QString savefilename =
              QFileDialog::getSaveFileName( QString::null,
                                            QString::null, this, filename);
        if ( !savefilename.isEmpty() )
           if ( !image.save( savefilename, fmt ) )
              QMessageBox::warning( this, "Save failed",
                                          "Error saving file" );
    }
    void savePixmap( int item ) {
       const char* fmt = savepixmap->text(item);
       QString savefilename =
            QFileDialog::getSaveFileName( QString::null,
                                          QString::null, this, filename);
       if ( !savefilename.isEmpty() )
          if ( !pm.save( savefilename, fmt ) )
             QMessageBox::warning( this, "Save failed",
                                         "Error saving file" );
    }

    void mouseMoved( int x, int y ) {
       this->x = x;
       this->y = y;
       updateStatus();
    }

private:
    void setMenuItemFlags() {
       bool valid_image = pm.size() != QSize( 0, 0 );
       file->setItemEnabled( si, valid_image );
       file->setItemEnabled( sp, valid_image );

       edit->setItemEnabled( t1, image.depth() != 1 );
       edit->setItemEnabled( t8, image.depth() != 8 );
       edit->setItemEnabled( t32, image.depth() != 32 );
    }

    void setImage(const QImage& newimage) {
       image = newimage;
       setCaption( filename );
       if ( image.width() == 0 )
          return;
       QApplication::setOverrideCursor( waitCursor );
       if ( pm.convertFromImage(image) )
          ms->setPixmap( pm );
       QApplication::restoreOverrideCursor();
       updateStatus();
       setMenuItemFlags();
    }

    void updateStatus() {
       if ( pm.size() == QSize( 0, 0 ) ) {
          if ( filename )
             status->setText("Could not load image");
          else
             status->setText("No image - select Open from File menu.");
       } else {
          QString message, moremsg;
          message.sprintf("%dx%d", image.width(), image.height());
          moremsg.sprintf(", %d Bits ", image.depth());
          message += moremsg;
          if (image.valid( x, y)) {
             moremsg.sprintf("(%d,%d)=#%0*x ", x, y,
                          image.hasAlphaBuffer() ? 8 : 6,
                          image.pixel( x, y ));
             message += moremsg;
          }
          if ( image.numColors() > 0 ) {
             if ( image.valid( x, y ))
                 moremsg.sprintf(", %d/%d colors", image.pixelIndex( x, y ),
                                                   image.numColors());
             else
                 moremsg.sprintf(", %d colors", image.numColors());
             message += moremsg;
          }
          status->setText(message);
       }
    }

    QString    filename;
    QImage     image;    // loaded image
    QPixmap    pm;       // converted pixmap

    MyScrollView *ms;
    QMenuBar     *menubar;
    QPopupMenu   *file, *saveimage, *savepixmap, *edit;

    QLabel      *status;
    int         x, y, si, sp,  t1, t8, t32;  // Menu Ids
};

//............................................... ImageViewer (Konstruktor)
ImageViewer::ImageViewer( QWidget *parent, const char *name, int wFlags )
                        : QWidget( parent, name, wFlags ), filename( 0 ) {
    x = y = -1;

    QBoxLayout *box = new QVBoxLayout( this, 0, 30 );

    menubar = new QMenuBar(this);
    menubar->setSeparator( QMenuBar::InWindowsStyle );

    QStrList fmt = QImage::outputFormats();
    saveimage = new QPopupMenu();
    savepixmap = new QPopupMenu();
    for (const char* f = fmt.first(); f; f = fmt.next()) {
       saveimage->insertItem( f );
       savepixmap->insertItem( f );
    }
    connect( saveimage, SIGNAL(activated(int)),
             this, SLOT(saveImage(int)) );
    connect( savepixmap, SIGNAL(activated(int)),
             this, SLOT(savePixmap(int)) );

    file = new QPopupMenu();
    menubar->insertItem( "&File", file );
    file->insertItem( "&Open...", this,  SLOT(openFile()), CTRL+Key_O );
    si = file->insertItem( "Save image", saveimage );
    sp = file->insertItem( "Save pixmap", savepixmap );
    file->insertSeparator();
    file->insertItem( "E&xit", qApp,  SLOT(quit()), CTRL+Key_Q );

    edit =  new QPopupMenu();
    menubar->insertItem( "&Edit", edit );
    edit->insertItem("&Horizontal flip", this, SLOT(hFlip()), ALT+Key_H);
    edit->insertItem("&Vertical flip", this, SLOT(vFlip()), ALT+Key_V);
    edit->insertItem("&Rotate 180", this, SLOT(rot180()), ALT+Key_R);
    edit->insertSeparator();
    edit->insertItem("&Swap RGB", this, SLOT(swapRgb()), ALT+Key_S);
    edit->insertItem("&Invert Pixels", this, SLOT(invertPix()), ALT+Key_I);
    edit->insertSeparator();
    t1 = edit->insertItem( "Convert to &1 bit", this, SLOT(to1Bit()) );
    t8 = edit->insertItem( "Convert to &8 bit", this, SLOT(to8Bit()) );
    t32 = edit->insertItem( "Convert to &32 bit", this, SLOT(to32Bit()) );

    box->addWidget( menubar );

    ms = new MyScrollView( this );
    box->addWidget( ms );
    QObject::connect( ms, SIGNAL( pos( int, int ) ),
                      this, SLOT( mouseMoved( int, int ) ) );

    status = new QLabel(this);
    status->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
    status->setFixedHeight( fontMetrics().height() + 4 );
    box->addWidget( status );
}

#include "image2.moc"

//.................................................................... main
int main( int argc, char *argv[] ) {
    QApplication::setColorSpec( QApplication::CustomColor );
    QApplication::setFont( QFont("Helvetica", 12) );
    QApplication a( argc, argv );

    QFileDialog::setIconProvider(new ImageIconProvider);

    ImageViewer *w = new ImageViewer(0, "new window",
                         Qt::WDestructiveClose | Qt::WResizeNoErase );
    w->show();
    QObject::connect(qApp, SIGNAL(lastWindowClosed()), qApp, SLOT(quit()));

    return a.exec();
}
