#include <qapplication.h>
#include <qnetwork.h>
#include <qvbox.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qfiledialog.h>
#include <qurloperator.h>
#include <qlayout.h>
#include <qprogressbar.h>

//......................................................... Klasse FtpProgress
class FtpProgress : public QWidget
{
    Q_OBJECT
public:
    FtpProgress( int files, int bytes,
                 QWidget *p=0, const char *t=0) : QWidget( p, t ) {
       QGridLayout *qvbox = new QGridLayout( this, 3, 2, 5 );

       QLabel *l = new QLabel( "Files copied", this );
       qvbox->addWidget( l, 0, 0 );

       fileProgress = new QProgressBar( files, this );
       fileProgress->setCenterIndicator( true );
       qvbox->addWidget( fileProgress, 0, 1 );

       l = new QLabel( "Bytes copied", this );
       qvbox->addWidget( l, 1, 0 );

       byteProgress = new QProgressBar( bytes, this );
       byteProgress->setCenterIndicator( true );
       qvbox->addWidget( byteProgress, 1, 1 );

       QPushButton *cancel = new QPushButton( "Cancel", this );
       connect( cancel, SIGNAL( clicked() ), SIGNAL( cancelled() ) );
       qvbox->addMultiCellWidget( cancel, 2, 2, 0, 1 );

       setCaption( "File Transfer in Progress" );
       resize( 400, 150 );
       show();
    }

    void setFileProgress( int files ) {
       fileProgress->setProgress( files );
    } 

    void setByteProgress( int bytes ) {
       byteProgress->setProgress( bytes );
    } 

signals:
    void cancelled( void );

private:
    QProgressBar *fileProgress,
                 *byteProgress;
};

//............................................................... class View
class View : public QVBox {
    Q_OBJECT
public:
    View() : QVBox() {
       setSpacing( 5 );
       setMargin( 5 );

       ftpProgress = 0;
       QPushButton *b = new QPushButton( "Download of Files...", this );
       connect( b,    SIGNAL( clicked() ),
                this, SLOT( downloadFile() ) );
       b->setFont( QFont( "Times", 24 ) );

       logInfo = new QLabel( this );
       logInfo->setAlignment( Qt::AlignCenter );
       logInfo->setPaletteBackgroundColor( Qt::white );
       logInfo->setFont( QFont( "Times", 18 ) );

       connect( &op, SIGNAL( newChildren( const QValueList<QUrlInfo> &,
                                          QNetworkOperation * ) ),
                this, SLOT( calculateSize( const QValueList<QUrlInfo> &,
                                           QNetworkOperation * ) ) );
       connect( &op,
                SIGNAL(startedNextCopy( const QPtrList<QNetworkOperation> &) ),
                this, SLOT( nextFileCopy() ) );

       connect( &op,
           SIGNAL( dataTransferProgress( int, int, QNetworkOperation * ) ),
           this, SLOT( downloadProgress( int, int, QNetworkOperation * ) ) );

       connect( &op,  SIGNAL( finished( QNetworkOperation * ) ),
                this, SLOT( done( QNetworkOperation * ) ) );
    }
    
private slots:
    void downloadFile() {
       if ( ftpProgress ) {
          delete ftpProgress;
          ftpProgress = 0;
       }
       totalSize = totalFiles = doneSize = doneFiles = 0;
       QString wd;
       from = getOpenFileNames( wd );
       if ( !from.isEmpty() ) {
           to = QFileDialog::getExistingDirectory( "." );
           if ( !to.isEmpty() ) {
              op = wd;
              totalFiles = from.count();
              op.listChildren();
           }
       }
    }

    void calculateSize( const QValueList<QUrlInfo> &info,
                        QNetworkOperation * ) {
       QValueList<QUrlInfo>::ConstIterator it = info.begin();
       for ( ; it != info.end(); ++it )
          for ( QStringList::Iterator it2 = from.begin(); 
                it2 != from.end(); ++it2 ) {
                QFileInfo fi( *it2 );
               if ( (*it).name() == fi.fileName() ) {
                   const QUrlInfo &inf = *it;
                   totalSize += inf.size(); 
                   break;
               }
          }
    }

    void nextFileCopy( void ) {
       ftpProgress->setFileProgress( ++doneFiles );
       QString t;
       t.sprintf( "%d. of %d Files currently copied\n"
                  "%d of %d Bytes copied\n",
                  doneFiles, totalFiles, doneSize, totalSize );
       oldDoneSize = doneSize;
       logInfo->setText( t );
    }

    void downloadProgress( int bytesDone, int, QNetworkOperation *o ) {
       if ( o->operation() == QNetworkProtocol::OpGet ) {
          doneSize = oldDoneSize + bytesDone;
          QString t;
          t.sprintf( "%d. of %d Files currently copied\n"
                     "%d of %d Bytes copied\n",
                     doneFiles, totalFiles, doneSize, totalSize );
          logInfo->setText( t );
          ftpProgress->setByteProgress( doneSize );
       }
    }

    void done( QNetworkOperation *o ) {
       if ( o->operation() == QNetworkProtocol::OpListChildren ) {
          ftpProgress = new FtpProgress( totalFiles, totalSize );
          connect( ftpProgress, SIGNAL( cancelled() ),
                   this,        SLOT( cancelFtp() ) );
          op.copy( from, to );
       } else if ( o->state() != QNetworkProtocol::StInProgress ) {
          QString t;
          t.sprintf( "%d. of %d Files currently copied\n"
                     "%d of %d Bytes copied\n",
                     doneFiles, totalFiles, doneSize, totalSize );
          if ( o->state() == QNetworkProtocol::StDone )
             t += " ... file transfer successful";
          else
             t += " ... file transfer not successful";
          logInfo->setText( t );
          if ( ftpProgress && doneFiles==totalFiles && doneSize==totalSize ) {
             delete ftpProgress;
             ftpProgress = 0;
          }
       }
    }

    void cancelFtp( void ) {
       op.stop();
       delete ftpProgress;
       ftpProgress = 0;
    }

private:
    FtpProgress     *ftpProgress;
    QLabel          *logInfo;
    QUrlOperator    op;
    QStringList     from;
    QString         to;
    int             totalSize, totalFiles,
                    oldDoneSize, doneSize,  doneFiles;
 
    QStringList getOpenFileNames( QString &wd ) {
       static QString workingDirectory = "ftp://ftp.suse.com/";

       QFileDialog *dlg = new QFileDialog( workingDirectory,
                                           QString::null, 0, 0, true );
       dlg->setCaption( "Open" );
       dlg->setMode( QFileDialog::ExistingFiles );
       QStringList result;
       if ( dlg->exec() == QDialog::Accepted ) {
          result = dlg->selectedFiles();
          workingDirectory = dlg->url();
       }
       wd = workingDirectory;
       delete dlg;
       return result;
    }
};

#include "multidownload.moc"

//..................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    qInitNetworkProtocols(); //... Initialisieren der implementierten
                             //... Netzwerkprotokolle
    View v;
    v.resize( 400, 300 );
    v.show();
    a.setMainWidget( &v );
    
    return a.exec();
}
