#include <qapplication.h>
#include <qnetwork.h>
#include <qvbox.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qtextedit.h>
#include <qfiledialog.h>
#include <qnetworkprotocol.h>
#include <qurloperator.h>
#include <qurlinfo.h>
#include <qsocket.h>
#include <qstringlist.h>
#include <qregexp.h>
//............................................................... class Nntp
class Nntp : public QNetworkProtocol {
    Q_OBJECT
public:
    Nntp() : QNetworkProtocol(), connectionReady( false ),
            readGroups( false ), readArticle( false ) {
       commandSocket = new QSocket( this );
       connect( commandSocket, SIGNAL(hostFound()), this, SLOT(hostFound()));
       connect( commandSocket, SIGNAL(connected()), this, SLOT(connected()));
       connect( commandSocket, SIGNAL(connectionClosed()),
                this,          SLOT(closed()));
       connect( commandSocket, SIGNAL(readyRead()), this, SLOT(readyRead()));
       connect( commandSocket, SIGNAL(error(int)), this, SLOT(error(int)));
    }
    ~Nntp() { close(); delete commandSocket;                         }
    int supportedOperations() const { return OpListChildren | OpGet; }
protected:
    virtual void operationListChildren( QNetworkOperation * ) {
       QString path = url()->path(), cmd;
       if ( path.isEmpty() || path == "/" ) {
          cmd = "list newsgroups\r\n";
       } else if ( url()->isDir() ) {
          path = path.replace( QRegExp( "/" ), "" );
          cmd = "listgroup " + path + "\r\n";
       } else
          return;
       commandSocket->writeBlock( cmd.latin1(), cmd.length() );
       readGroups = true;
    }
    virtual void operationGet( QNetworkOperation *op ) {
       QUrl u( op->arg( 0 ) );
       QString dirPath = u.dirPath(), file = u.fileName();
       dirPath = dirPath.replace( QRegExp( "/" ), "" );
       QString cmd;
       cmd = "group " + dirPath + "\r\n";
       commandSocket->writeBlock( cmd.latin1(), cmd.length() );
       cmd = "article " + file + "\r\n"; // read the head of the article
       commandSocket->writeBlock( cmd.latin1(), cmd.length() );
       readArticle = true;
    }
private:
    QSocket *commandSocket;
    bool     connectionReady, readGroups, readArticle;
    bool checkConnection( QNetworkOperation * ) {
       if ( commandSocket->isOpen() )
          return connectionReady;
       if ( commandSocket->state() == QSocket::Connecting )
          return false;
       commandSocket->connectToHost( url()->host(),
                                  url()->port() != -1 ? url()->port() : 119 );
       return connectionReady = false;
    }
    void close() {
       if ( commandSocket->isOpen() ) {
          commandSocket->writeBlock( "quit\r\n", strlen( "quit\r\n" ) );
          commandSocket->close();
       }
    }
    void parseGroups() {
       if ( !commandSocket->canReadLine() )
          return;
       while ( commandSocket->canReadLine() ) {
          QString s = commandSocket->readLine();
          if ( s[ 0 ] == '.' ) {
             readGroups = false;
             operationInProgress()->setState( StDone );
             emit finished( operationInProgress() );
             return;
          }
          if ( s.left( 3 ) == "215" || s.left( 3 ) == "211" ) {
             operationInProgress()->setState( StInProgress );
             emit start( operationInProgress() );
             continue;
          }
          bool tab = s.find( '\t' ) != -1;
          QString group = s.mid( 0, s.find( tab ? '\t' : ' ' ) );
          QUrlInfo inf;
          inf.setName( group );
          QString path = url()->path();
          inf.setDir( path.isEmpty() || path == "/" );
          inf.setSymLink( false );
          inf.setFile( !inf.isDir() );
          inf.setWritable( false );
          inf.setReadable( true );
          emit newChild( inf, operationInProgress() );
       }
    }
    void parseArticle() {
       if ( !commandSocket->canReadLine() )
          return;
       while ( commandSocket->canReadLine() ) {
          QString s = commandSocket->readLine();
          if ( s[ 0 ] == '.' ) {
             readArticle = false;
             operationInProgress()->setState( StDone );
             emit finished( operationInProgress() );
             return;
          }
          if ( s.right( 1 ) == "\n" )
             s.remove( s.length() - 1, 1 );
          emit data( QCString( s.ascii() ), operationInProgress() );
       }
    }
protected slots:
    void hostFound() {
       emit connectionStateChanged( ConHostFound,
                url() ? tr( "Host %1 found" ).arg( url()->host() )
                      : tr( "Host found" ) );
    }
    void connected() {
       emit connectionStateChanged( ConConnected,
              url() ? tr( "Connected to host %1" ).arg( url()->host() )
                    : tr( "Connected to host" ) );
    }
    void closed() {
       emit connectionStateChanged( ConClosed,
              url() ? tr( "Connection to %1 closed" ).arg( url()->host() )
                    : tr( "Connection closed" ) );
    }
    void readyRead() {
       if ( readGroups )  { parseGroups(); return; }
       if ( readArticle ) { parseArticle(); return; }

       QCString s;
       s.resize( commandSocket->bytesAvailable() );
       commandSocket->readBlock( s.data(), commandSocket->bytesAvailable() );

       if ( !url() )
          return;
       if ( s.left( 3 ) == "200" )
          connectionReady = true;
    }
    void error( int code ) {
       if ( code == QSocket::ErrHostNotFound ||
          code == QSocket::ErrConnectionRefused ) {
          if ( operationInProgress() ) {
             QString msg = tr( "Host not found or couldn't connect to: \n" +
                           url()->host() );
             operationInProgress()->setState( StFailed );
             operationInProgress()->setProtocolDetail( msg );
             operationInProgress()->setErrorCode( (int)ErrHostNotFound );
             clearOperationQueue();
             emit finished( operationInProgress() );
          }
       }
    }
};
//............................................................... class View
class View : public QVBox {
    Q_OBJECT
public:
    View() : QVBox() {
       setSpacing( 5 );
       setMargin( 5 );
       QPushButton *b = new QPushButton( "Download a file...", this );
       connect( b,    SIGNAL( clicked() ),
                this, SLOT( downloadFile() ) );
       fileView = new QTextEdit( this );
       fileView->setReadOnly( true );
       connect( &op,  SIGNAL( data( const QByteArray &, QNetworkOperation * ) ),
                this, SLOT( newData( const QByteArray & ) ) );
    }
private slots:
    void downloadFile() {
       QString file = getOpenFileName();
       if ( !file.isEmpty() ) {
          fileView->clear();
          op = file;    // Herunterladen der Daten
          op.get();
       }
    }
    void newData( const QByteArray &ba ) { fileView->append( ba ); }
private:
    QTextEdit    *fileView;
    QUrlOperator op;
    QString getOpenFileName() {
       static QString workingDirectory = "ftp://ftp.suse.com/";
       QFileDialog *dlg = new QFileDialog( workingDirectory,
                                           QString::null, 0, 0, true );
       dlg->setCaption( "Open" );
       dlg->setMode( QFileDialog::ExistingFile );
       QString result;
       if ( dlg->exec() == QDialog::Accepted ) {
          result = dlg->selectedFile();
          workingDirectory = dlg->url();
       }
       delete dlg;
       return result;
    }
};
#include "networkprotocol.moc"
//..................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    qInitNetworkProtocols();  // Initial. der implementierten Netzwerkprotokolle
    QNetworkProtocol::registerNetworkProtocol( "nntp",
                                       new QNetworkProtocolFactory<Nntp> );
    View v;
    v.resize( 600, 600 );
    v.show();
    a.setMainWidget( &v );
    return a.exec();
}
