#include <qnp.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qtextview.h>
#include <qfile.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <stdio.h>

#define MAX_LINES  10000

//.............................................................. TextWidget
class TextWidget : public QTextView {
public:
   TextWidget( QWidget* p=0, const char* n=0 ) : QTextView( p, n ) {
      lineNo = 0;
      expr = "";
   }

   void addLine( QString &text ) {
      lines[lineNo++] = text; 
   }

   void setExpression( QString expression ) {
      expr = expression;
      showText();
   }

   void showText() {
      QRegExp r( expr );
      QString text;
      QString line;
      for ( int i=0; i<lineNo; i++) {
         if ( expr.isEmpty() || lines[i].contains( r ) ) {
            if ( withLineNo )
               line.sprintf("%6d %s", i+1, (const char *)lines[i] );
            else
               line.sprintf("%s", (const char *)lines[i] );
            text += line;
         }
      }
      this->setText( text );
   }
   void setLineNo( bool value )   { withLineNo = value; }

private:
   int       lineNo;
   QString   lines[MAX_LINES]; 
   QString   expr;
   bool      withLineNo;
};

//.............................................................. NetsWidget
class NetsWidget : public QNPWidget {
   Q_OBJECT
public:
   NetsWidget() {
      resize( 800, 220 );

      textShow = new TextWidget( this );
      textShow->setGeometry( 0, 40, width(), height()-40 );
      textShow->setFont( QFont( "Courier", 12 ) ); 

      grepInput = new QLineEdit( this );
      grepInput->setGeometry( 0, 0, 150, 40 );
      connect( grepInput, SIGNAL( returnPressed() ),
               this,      SLOT( newExpression() ) );

      buttonBox1 = new QButtonGroup( 2, Horizontal, "LineNumbers ?", this);
      buttonBox1->setGeometry( 180, 0, 150, 40 );
      buttonBox1->setPalette( QPalette( Qt::yellow, Qt::yellow ) );
      buttonBox1->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
      QRadioButton *yes = new QRadioButton( "yes", buttonBox1 );
      QRadioButton *no  = new QRadioButton( "no",  buttonBox1 );
      yes->setChecked( true );
      textShow->setLineNo( true );
      connect( yes, SIGNAL( clicked() ), this, SLOT( withNo() ) );
      connect( no,  SIGNAL( clicked() ), this, SLOT( withOutNo() ) );
   }

   void showText()               { textShow->showText();      }
   void addLine( QString &text ) { textShow->addLine( text ); }

   void resizeEvent( QResizeEvent *event ) {
      textShow->setGeometry( 0, grepInput->height(),
                             event->size().width(),
                             event->size().height() - grepInput->height() );
      grepInput->setGeometry( 0, 0, 150, 40 );
      buttonBox1->setGeometry( 180, 0, 150, 40 );
   }

   void mousePressEvent(QMouseEvent *) {
      QMessageBox::about( this, "Nsgrep",
                    "Qt-basierendes Plugin, das interaktives grep ermoeglicht" );
   }

private slots:
   void newExpression( void ) {
      textShow->setExpression( grepInput->text() );
   }
   void withNo( void )      { textShow->setLineNo( true );    showText(); }
   void withOutNo( void )   { textShow->setLineNo( false );   showText(); }

private:
   TextWidget   *textShow;
   QLineEdit    *grepInput;
   QButtonGroup *buttonBox1,
                *buttonBox2;
};

//.............................................................. TextLoader
class TextLoader : public QNPInstance 
{
public:
   TextLoader() { file = 0; }
  
   QNPWidget* newWindow() {
      view = new NetsWidget;
      return view;
   }

   void loadFile() {
      if (!file)
         return;
      int  r;
      QString line;
      while ( ( r = file->readLine( line, 10000 ) ) )  {
         if ( r == -1 ) {
            if ( !file->atEnd() ) {
               QString msg;
               msg.sprintf( "Fehler beim Lesen der Datei `%s'",
                            (const char *)fileName );
               QMessageBox::information( 0, "Fehler beim Lesen",
                                         msg, QMessageBox::Ok );
               return;
            } else
               break;
         } else
            view->addLine(line);
      }
      file->close();
      view->showText();
   }
  
   bool newStreamCreated( QNPStream*, StreamMode& smode ) {
      smode = AsFileOnly;
      return true;
   }
  
   void streamAsFile( QNPStream*, const char* fName ) {
      file = new QFile( fileName = fName );
      if ( !file->open( IO_ReadOnly ) )
         debug( "Could not open file: %s\n", fName );
      else
         loadFile();
   }

private:
   QString      fileName;
   QFile        *file;
   NetsWidget   *view;
};

//.......................................................... GrepTextPlugin
class GrepTextPlugin : public QNPlugin 
{
public:
   GrepTextPlugin() {}

   QNPInstance* newInstance() { return new TextLoader; }

   const char *getMIMEDescription() const {
      return "Nsgrep/mygrep:grp:Grep interaktiv";
   }

   const char *getPluginNameString() const {
      return "Interaktives Grep Plugin";
   }

   const char *getPluginDescriptionString() const {
      return "Interaktives Grep mit Eingabe von regulren Ausdrcken";
   }
};

//...................................................... QNPlugin::create()
QNPlugin* QNPlugin::create() {
    return new GrepTextPlugin;
}

#include "nsgrep.moc"
